@@ -1,9 +1,19 @@
+# This file is part of h5py, a Python interface to the HDF5 library.
+#
+# http://www.h5py.org
+#
+# Copyright 2008-2013 Andrew Collette and contributors
+#
+# License:  Standard 3-clause BSD; see "license.txt" for full license terms
+#           and contributor agreement.
+
 import posixpath as pp
 import sys
 import numpy
 
+import h5py
 from h5py import h5s, h5t, h5r, h5d
-from .base import HLObject
+from .base import HLObject, py3
 from . import filters
 from . import selections as sel
 from . import selections2 as sel2
@@ -30,15 +40,16 @@ def readtime_dtype(basetype, names):
 def make_new_dset(parent, shape=None, dtype=None, data=None,
                  chunks=None, compression=None, shuffle=None,
                     fletcher32=None, maxshape=None, compression_opts=None,
-                  fillvalue=None):
-    """ Return a new low-level dataset identifier 
+                  fillvalue=None, scaleoffset=None, track_times=None):
+    """ Return a new low-level dataset identifier
 
     Only creates anonymous datasets.
     """
 
     # Convert data to a C-contiguous ndarray
     if data is not None:
-        data = numpy.asarray(data, order="C")
+        import base
+        data = numpy.asarray(data, order="C", dtype=base.guess_dtype(data))
 
     # Validate shape
     if shape is None:
@@ -50,16 +61,22 @@ def make_new_dset(parent, shape=None, dtype=None, data=None,
         if data is not None and (numpy.product(shape) != numpy.product(data.shape)):
             raise ValueError("Shape tuple is incompatible with data")
 
-    # Validate dtype
-    if dtype is None and data is None:
-        dtype = numpy.dtype("=f4")
-    elif dtype is None and data is not None:
-        dtype = data.dtype
+    if isinstance(dtype, h5py.Datatype):
+        # Named types are used as-is
+        tid = dtype.id
+        dtype = tid.dtype  # Following code needs this
     else:
-        dtype = numpy.dtype(dtype)
+        # Validate dtype
+        if dtype is None and data is None:
+            dtype = numpy.dtype("=f4")
+        elif dtype is None and data is not None:
+            dtype = data.dtype
+        else:
+            dtype = numpy.dtype(dtype)
+        tid = h5t.py_create(dtype, logical=1)
 
     # Legacy
-    if any((compression, shuffle, fletcher32, maxshape)) and chunks is False:
+    if any((compression, shuffle, fletcher32, maxshape,scaleoffset)) and chunks is False:
         raise ValueError("Chunked format required for given storage options")
 
     # Legacy
@@ -76,16 +93,21 @@ def make_new_dset(parent, shape=None, dtype=None, data=None,
         compression = 'gzip'
 
     dcpl = filters.generate_dcpl(shape, dtype, chunks, compression, compression_opts,
-                  shuffle, fletcher32, maxshape)
+                  shuffle, fletcher32, maxshape, scaleoffset)
 
     if fillvalue is not None:
         fillvalue = numpy.array(fillvalue)
         dcpl.set_fill_value(fillvalue)
 
+    if track_times in (True, False):
+        dcpl.set_obj_track_times(track_times)
+    elif track_times is not None:
+        raise TypeError("track_times must be either True or False")
+
     if maxshape is not None:
         maxshape = tuple(m if m is not None else h5s.UNLIMITED for m in maxshape)
     sid = h5s.create_simple(shape, maxshape)
-    tid = h5t.py_create(dtype, logical=1)
+
 
     dset_id = h5d.create(parent.id, None, tid, sid, dcpl=dcpl)
 
@@ -94,29 +116,52 @@ def make_new_dset(parent, shape=None, dtype=None, data=None,
 
     return dset_id
 
-class _RegionProxy(object):
 
-    def __init__(self, dset):
-        self.id = dset.id
+class AstypeContext(object):
+
+    def __init__(self, dset, dtype):
+        self._dset = dset
+        self._dtype = numpy.dtype(dtype)
+
+    def __enter__(self):
+        self._dset._local.astype = self._dtype
+
+    def __exit__(self, *args):
+        self._dset._local.astype = None
 
-    def __getitem__(self, args):
-        selection = sel.select(self.id.shape, args, dsid=self.id)
-        return h5r.create(self.id, b'.', h5r.DATASET_REGION, selection._id)
 
 class Dataset(HLObject):
 
     """
         Represents an HDF5 dataset
     """
+        
+    def astype(self, dtype):
+        """ Get a context manager allowing you to perform reads to a
+        different destination type, e.g.:
 
-    def _g_shape(self):
+        >>> with dataset.astype('f8'):
+        ...     double_precision = dataset[0:100:2]
+        """
+        return AstypeContext(self, dtype)
+
+    @property
+    def dims(self):
+        from . dims import DimensionManager
+        return DimensionManager(self)
+
+    @property
+    def shape(self):
         """Numpy-style shape tuple giving dataset dimensions"""
         return self.id.shape
-
-    def _s_shape(self, shape):
+    @shape.setter
+    def shape(self, shape):
         self.resize(shape)
 
-    shape = property(_g_shape, _s_shape)
+    @property
+    def size(self):
+        """Numpy-style attribute giving the total dataset size"""
+        return numpy.prod(self.shape)
 
     @property
     def dtype(self):
@@ -126,6 +171,8 @@ class Dataset(HLObject):
     @property
     def value(self):
         """  Alias for dataset[()] """
+        DeprecationWarning("dataset.value has been deprecated. "
+            "Use dataset[()] instead.")
         return self[()]
 
     @property
@@ -158,7 +205,18 @@ class Dataset(HLObject):
     def fletcher32(self):
         """Fletcher32 filter is present (T/F)"""
         return 'fletcher32' in self._filters
-        
+
+    @property
+    def scaleoffset(self):
+        """Scale/offset filter settings. For integer data types, this is
+        the number of bits stored, or 0 for auto-detected. For floating
+        point data types, this is the number of decimal places retained.
+        If the scale/offset filter is not in use, this is None."""
+        try:
+            return self._filters['scaleoffset'][1]
+        except KeyError:
+            return None
+
     @property
     def maxshape(self):
         """Shape up to which this dataset can be resized.  Axes with value
@@ -174,22 +232,19 @@ class Dataset(HLObject):
         dcpl = self._dcpl.get_fill_value(arr)
         return arr[0]
 
-    @property
-    def regionref(self):
-        """Create a region reference.  The syntax is regionref[<slices>].
-        For example, dset.regionref[...] creates a region reference in which
-        the whole dataset is selected.
-        """
-        return _RegionProxy(self)
-
     def __init__(self, bind):
         """ Create a new Dataset object by binding to a low-level DatasetID.
         """
+        from threading import local
 
+        if not isinstance(bind, h5d.DatasetID):
+            raise ValueError("%s is not a DatasetID" % bind)
         HLObject.__init__(self, bind)
 
         self._dcpl = self.id.get_create_plist()
         self._filters = filters.get_filters(self._dcpl)
+        self._local = local()
+        self._local.astype = None
 
     def resize(self, size, axis=None):
         """ Resize the dataset, or the specified axis.
@@ -233,7 +288,7 @@ class Dataset(HLObject):
         return size
 
     def len(self):
-        """ The size of the first axis.  TypeError if scalar. 
+        """ The size of the first axis.  TypeError if scalar.
 
         Use of this method is preferred to len(dset), as Python's built-in
         len() cannot handle values greater then 2**32 on 32-bit systems.
@@ -272,12 +327,30 @@ class Dataset(HLObject):
         names = tuple(x for x in args if isinstance(x, str))
         args = tuple(x for x in args if not isinstance(x, str))
 
+        def strip_fields(basetype):
+            """ Strip extra dtype information from special types """
+            if basetype.kind == 'O':
+                return numpy.dtype('O')
+            if basetype.fields is not None:
+                if basetype.kind in ('i','u'):
+                    return basetype.fields['enum'][0]
+                fields = []
+                for name in basetype.names:
+                    fff = basetype.fields[name]
+                    if len(fff) == 3:
+                        (subtype, offset, meta) = fff
+                    else:
+                        subtype, meta = fff
+                        offset = 0
+                    subtype = strip_fields(subtype)
+                    fields.append((name, subtype))
+                return numpy.dtype(fields)
+            return basetype
+
         def readtime_dtype(basetype, names):
             """ Make a NumPy dtype appropriate for reading """
 
-            if basetype.kind == 'O':
-                # Special-dtype fields break indexing
-                basetype = numpy.dtype('O')
+            basetype = strip_fields(basetype)
 
             if len(names) == 0:  # Not compound, or we want all fields
                 return basetype
@@ -291,11 +364,41 @@ class Dataset(HLObject):
 
             return numpy.dtype([(name, basetype.fields[name][0]) for name in names])
 
-        # This is necessary because in the case of array types, NumPy
-        # discards the array information at the top level.
-        new_dtype = readtime_dtype(self.id.dtype, names)
+        if self._local.astype is not None:
+            new_dtype = readtime_dtype(self._local.astype, names)
+        else:
+            # This is necessary because in the case of array types, NumPy
+            # discards the array information at the top level.
+            new_dtype = readtime_dtype(self.id.dtype, names)
         mtype = h5t.py_create(new_dtype)
 
+        # === Special-case region references ====
+
+        if len(args) == 1 and isinstance(args[0], h5r.RegionReference):
+
+            obj = h5r.dereference(args[0], self.id)
+            if obj != self.id:
+                raise ValueError("Region reference must point to this dataset")
+
+            sid = h5r.get_region(args[0], self.id)
+            mshape = sel.guess_shape(sid)
+            if mshape is None:
+                return np.array((0,), dtype=new_dtype)
+            if numpy.product(mshape) == 0:
+                return np.array(mshape, dtype=new_dtype)
+            out = numpy.empty(mshape, dtype=new_dtype)
+            sid_out = h5s.create_simple(mshape)
+            sid_out.select_all()
+            self.id.read(sid_out, sid, out, mtype)
+            return out
+
+        # === Check for zero-sized datasets =====
+
+        if numpy.product(self.shape) == 0:
+            # These are the only access methods NumPy allows for such objects
+            if args == (Ellipsis,) or args == tuple():
+                return numpy.empty(self.shape, dtype=new_dtype)
+            
         # === Scalar dataspaces =================
 
         if self.shape == ():
@@ -304,6 +407,8 @@ class Dataset(HLObject):
             arr = numpy.ndarray(selection.mshape, dtype=new_dtype)
             for mspace, fspace in selection:
                 self.id.read(mspace, fspace, arr, mtype)
+            if len(names) == 1:
+                arr = arr[names[0]]
             if selection.mshape is None:
                 return arr[()]
             return arr
@@ -314,9 +419,9 @@ class Dataset(HLObject):
         selection = sel.select(self.shape, args, dsid=self.id)
 
         if selection.nselect == 0:
-            return numpy.ndarray((0,), dtype=new_dtype)
+            return numpy.ndarray(selection.mshape, dtype=new_dtype)
 
-        # Up-converting to (1,) so that numpy.ndarray correctly creates 
+        # Up-converting to (1,) so that numpy.ndarray correctly creates
         # np.void rows in case of multi-field dtype. (issue 135)
         single_element = selection.mshape == ()
         mshape = (1,) if single_element else selection.mshape
@@ -355,24 +460,66 @@ class Dataset(HLObject):
         names = tuple(x for x in args if isinstance(x, str))
         args = tuple(x for x in args if not isinstance(x, str))
 
-        if len(names) != 0:
-            raise TypeError("Field name selections are not allowed for write.")
-
         # Generally we try to avoid converting the arrays on the Python
         # side.  However, for compound literals this is unavoidable.
-        if self.dtype.kind == 'V' and \
-        (not isinstance(val, numpy.ndarray) or val.dtype.kind != 'V'):
-            val = numpy.asarray(val, dtype=self.dtype, order='C')
+        if self.dtype.kind == "O" or \
+          (self.dtype.kind == 'V' and \
+          (not isinstance(val, numpy.ndarray) or val.dtype.kind != 'V') and \
+          (self.dtype.subdtype == None)):
+            if len(names) == 1 and self.dtype.fields is not None:
+                # Single field selected for write, from a non-array source
+                if not names[0] in self.dtype.fields:
+                    raise ValueError("No such field for indexing: %s" % names[0])
+                dtype = self.dtype.fields[names[0]][0]
+                cast_compound = True
+            else:
+                dtype = self.dtype
+                cast_compound = False
+
+            val = numpy.asarray(val, dtype=dtype, order='C')
+            if cast_compound:
+                val = val.astype(numpy.dtype([(names[0], dtype)]))
         else:
             val = numpy.asarray(val, order='C')
 
         # Check for array dtype compatibility and convert
         if self.dtype.subdtype is not None:
             shp = self.dtype.subdtype[1]
-            if val.shape[-len(shp):] != shp:
-                raise TypeError("Can't broadcast to array dimension %s" % (shp,))
+            valshp = val.shape[-len(shp):]
+            if valshp != shp:  # Last dimension has to match
+                raise TypeError("When writing to array types, last N dimensions have to match (got %s, but should be %s)" % (valshp, shp,))
             mtype = h5t.py_create(numpy.dtype((val.dtype, shp)))
             mshape = val.shape[0:len(val.shape)-len(shp)]
+
+        # Make a compound memory type if field-name slicing is required
+        elif len(names) != 0:
+
+            mshape = val.shape
+
+            # Catch common errors
+            if self.dtype.fields is None:
+                raise TypeError("Illegal slicing argument (not a compound dataset)")
+            mismatch = [x for x in names if x not in self.dtype.fields]
+            if len(mismatch) != 0:
+                mismatch = ", ".join('"%s"'%x for x in mismatch)
+                raise ValueError("Illegal slicing argument (fields %s not in dataset type)" % mismatch)
+        
+            # Write non-compound source into a single dataset field
+            if len(names) == 1 and val.dtype.fields is None:
+                subtype = h5y.py_create(val.dtype)
+                mtype = h5t.create(h5t.COMPOUND, subtype.get_size())
+                mtype.insert(self._e(names[0]), 0, subtype)
+
+            # Make a new source type keeping only the requested fields
+            else:
+                fieldnames = [x for x in val.dtype.names if x in names] # Keep source order
+                mtype = h5t.create(h5t.COMPOUND, val.dtype.itemsize)
+                for fieldname in fieldnames:
+                    subtype = h5t.py_create(val.dtype.fields[fieldname][0])
+                    offset = val.dtype.fields[fieldname][1]
+                    mtype.insert(self._e(fieldname), offset, subtype)
+
+        # Use mtype derived from array (let DatasetID.write figure it out)
         else:
             mshape = val.shape
             mtype = None
@@ -386,7 +533,7 @@ class Dataset(HLObject):
         # Broadcast scalars if necessary.
         if (mshape == () and selection.mshape != ()):
             if self.dtype.subdtype is not None:
-                raise NotImplementedError("Scalar broadcasting is not supported for array dtypes")
+                raise TypeError("Scalar broadcasting is not supported for array dtypes")
             val2 = numpy.empty(selection.mshape[-1], dtype=val.dtype)
             val2[...] = val
             val = val2
@@ -411,6 +558,7 @@ class Dataset(HLObject):
 
         Broadcasting is supported for simple indexing.
         """
+
         if source_sel is None:
             source_sel = sel.SimpleSelection(self.shape)
         else:
@@ -453,19 +601,25 @@ class Dataset(HLObject):
         you have to read the whole dataset everytime this method is called.
         """
         arr = numpy.empty(self.shape, dtype=self.dtype if dtype is None else dtype)
+
+        # Special case for (0,)*-shape datasets
+        if numpy.product(self.shape) == 0:
+            return arr
+
         self.read_direct(arr)
         return arr
 
     def __repr__(self):
         if not self:
-            return "<Closed HDF5 dataset>"
-        if self.name is None:
-            namestr = '("anonymous")'
+            r = u'<Closed HDF5 dataset>'
         else:
-            name = pp.basename(pp.normpath(self.name))
-            namestr = '"%s"' % (name if name != '' else '/')
-        return '<HDF5 dataset %s: shape %s, type "%s">' % \
-            (namestr, self.shape, self.dtype.str)
-
-
-
+            if self.name is None:
+                namestr = u'("anonymous")'
+            else:
+                name = pp.basename(pp.normpath(self.name))
+                namestr = u'"%s"' % (name if name != u'' else u'/')
+            r = u'<HDF5 dataset %s: shape %s, type "%s">' % \
+                (namestr, self.shape, self.dtype.str)
+        if py3:
+            return r
+        return r.encode('utf8')
