@@ -1,5 +1,15 @@
+# This file is part of h5py, a Python interface to the HDF5 library.
+#
+# http://www.h5py.org
+#
+# Copyright 2008-2013 Andrew Collette and contributors
+#
+# License:  Standard 3-clause BSD; see "license.txt" for full license terms
+#           and contributor agreement.
+
 import posixpath as pp
 
+from ..h5t import TypeID
 from .base import HLObject
 
 class Datatype(HLObject):
@@ -22,6 +32,8 @@ class Datatype(HLObject):
     def __init__(self, bind):
         """ Create a new Datatype object by binding to a low-level TypeID.
         """
+        if not isinstance(bind, TypeID):
+            raise ValueError("%s is not a TypeID" % bind)
         HLObject.__init__(self, bind)
 
     def __repr__(self):
