@@ -1,17 +1,34 @@
+# This file is part of h5py, a Python interface to the HDF5 library.
+#
+# http://www.h5py.org
+#
+# Copyright 2008-2013 Andrew Collette and contributors
+#
+# License:  Standard 3-clause BSD; see "license.txt" for full license terms
+#           and contributor agreement.
+
+"""
+    File object test module.
+
+    Tests all aspects of File objects, including their creation.
+"""
+
 from __future__ import with_statement
 
 import os
 
-from .common import ut, TestCase
+from .common import ut, TestCase, unicode_filenames
 from h5py.highlevel import File
+import h5py
 
+mpi = h5py.get_config().mpi
 
 class TestFileOpen(TestCase):
 
     """
         Feature: Opening files with Python-style modes.
     """
-            
+
     def test_create(self):
         """ Mode 'w' opens file in overwrite mode """
         fname = self.mktemp()
@@ -49,7 +66,7 @@ class TestFileOpen(TestCase):
             self.assert_('bar' in fid)
         finally:
             fid.close()
-            
+
     def test_readonly(self):
         """ Mode 'r' opens file in readonly mode """
         fname = self.mktemp()
@@ -73,7 +90,7 @@ class TestFileOpen(TestCase):
         fid.create_group('bar')
         self.assert_('bar' in fid)
         fid.close()
-        
+
     def test_nonexistent_file(self):
         """ Modes 'r' and 'r+' do not create files """
         fname = self.mktemp()
@@ -87,6 +104,12 @@ class TestFileOpen(TestCase):
         with self.assertRaises(ValueError):
             File(self.mktemp(), 'mongoose')
 
+class TestModes(TestCase):
+
+    """
+        Feature: File mode can be retrieved via file.mode
+    """
+
     def test_mode_attr(self):
         """ Mode equivalent can be retrieved via property """
         fname = self.mktemp()
@@ -95,13 +118,41 @@ class TestFileOpen(TestCase):
         with File(fname, 'r') as f:
             self.assertEqual(f.mode, 'r')
 
+    def test_mode_external(self):
+        """ Mode property works for files opened via external links
+
+        Issue 190.
+        """
+        fname1 = self.mktemp()
+        fname2 = self.mktemp()
+
+        f1 = File(fname1,'w')
+        f1.close()
+
+        f2 = File(fname2,'w')
+        try:
+            f2['External'] = h5py.ExternalLink(fname1, '/')
+            f3 = f2['External'].file
+            self.assertEqual(f3.mode, 'r+')
+        finally:
+            f2.close()
+            f3.close()
+
+        f2 = File(fname2,'r')
+        try:
+            f3 = f2['External'].file
+            self.assertEqual(f3.mode, 'r')
+        finally:
+            f2.close()
+            f3.close()
+
 class TestDrivers(TestCase):
 
     """
         Feature: Files can be opened with low-level HDF5 drivers
     """
 
-    @ut.skipUnless(os.name, 'posix')
+    @ut.skipUnless(os.name == 'posix', "Stdio driver is supported on posix")
     def test_stdio(self):
         """ Stdio driver is supported on posix """
         fid = File(self.mktemp(), 'w', driver='stdio')
@@ -109,7 +160,7 @@ class TestDrivers(TestCase):
         self.assertEqual(fid.driver, 'stdio')
         fid.close()
 
-    @ut.skipUnless(os.name, 'posix')
+    @ut.skipUnless(os.name == 'posix', "Sec2 driver is supported on posix")
     def test_sec2(self):
         """ Sec2 driver is supported on posix """
         fid = File(self.mktemp(), 'w', driver='sec2')
@@ -148,7 +199,7 @@ class TestDrivers(TestCase):
         with self.assertRaises(ValueError):
             fid.create_group('bar')
         fid.close()
-    
+
     def test_blocksize(self):
         """ Core driver supports variable block size """
         fname = self.mktemp()
@@ -157,6 +208,27 @@ class TestDrivers(TestCase):
         self.assert_(fid)
         fid.close()
 
+    @ut.skipUnless(mpi, "Parallel HDF5 is required for MPIO driver test")
+    def test_mpio(self):
+        """ MPIO driver and options """
+        from mpi4py import MPI
+
+        fname = self.mktemp()
+        with File(fname, 'w', driver='mpio', comm=MPI.COMM_WORLD) as f:
+            self.assertTrue(f)
+            self.assertEqual(f.driver, 'mpio')
+
+    @ut.skipUnless(mpi, "Parallel HDF5 required")
+    def test_mpi_atomic(self):
+        """ Enable atomic mode for MPIO driver """
+        from mpi4py import MPI
+
+        fname = self.mktemp()
+        with File(fname, 'w', driver='mpio', comm=MPI.COMM_WORLD) as f:
+            self.assertFalse(f.atomic)
+            f.atomic = True
+            self.assertTrue(f.atomic)
+
     #TODO: family driver tests
 
 class TestLibver(TestCase):
@@ -184,6 +256,92 @@ class TestLibver(TestCase):
         self.assertEqual(f.libver, ('earliest', 'latest'))
         f.close()
 
+class TestUserblock(TestCase):
+
+    """
+        Feature: Files can be create with user blocks
+    """
+
+    def test_create_blocksize(self):
+        """ User blocks created with w, w- and properties work correctly """
+        f = File(self.mktemp(),'w-', userblock_size=512)
+        try:
+            self.assertEqual(f.userblock_size, 512)
+        finally:
+            f.close()
+
+        f = File(self.mktemp(),'w', userblock_size=512)
+        try:
+            self.assertEqual(f.userblock_size, 512)
+        finally:
+            f.close()
+
+    def test_write_only(self):
+        """ User block only allowed for write """
+        name = self.mktemp()
+        f = File(name, 'w')
+        f.close()
+
+        with self.assertRaises(ValueError):
+            f = h5py.File(name, 'r', userblock_size=512)
+
+        with self.assertRaises(ValueError):
+            f = h5py.File(name, 'r+', userblock_size=512)
+
+    def test_match_existing(self):
+        """ User block size must match that of file when opening for append """
+        name = self.mktemp()
+        f = File(name, 'w', userblock_size=512)
+        f.close()
+
+        with self.assertRaises(ValueError):
+            f = File(name, 'a', userblock_size=1024)
+
+        f = File(name, 'a', userblock_size=512)
+        try:
+            self.assertEqual(f.userblock_size, 512)
+        finally:
+            f.close()
+
+    def test_power_of_two(self):
+        """ User block size must be a power of 2 and at least 512 """
+        name = self.mktemp()
+
+        with self.assertRaises(ValueError):
+            f = File(name, 'w', userblock_size=128)
+
+        with self.assertRaises(ValueError):
+            f = File(name, 'w', userblock_size=513)
+
+        with self.assertRaises(ValueError):
+            f = File(name, 'w', userblock_size=1023)
+
+    def test_write_block(self):
+        """ Test that writing to a user block does not destroy the file """
+        name = self.mktemp()
+
+        f = File(name, 'w', userblock_size=512)
+        f.create_group("Foobar")
+        f.close()
+
+        pyfile = open(name, 'r+b')
+        try:
+            pyfile.write(b'X'*512)
+        finally:
+            pyfile.close()
+
+        f = h5py.File(name, 'r')
+        try:
+            self.assert_("Foobar" in f)
+        finally:
+            f.close()
+
+        pyfile = open(name, 'rb')
+        try:
+            self.assertEqual(pyfile.read(512), b'X'*512)
+        finally:
+            pyfile.close()
+
 class TestContextManager(TestCase):
 
     """
@@ -202,6 +360,7 @@ class TestUnicode(TestCase):
         Feature: Unicode filenames are supported
     """
 
+    @ut.skipIf(not unicode_filenames, "Filesystem unicode support required")
     def test_unicode(self):
         """ Unicode filenames can be used, and retrieved properly via .filename
         """
@@ -212,7 +371,7 @@ class TestUnicode(TestCase):
             self.assertIsInstance(fid.filename, unicode)
         finally:
             fid.close()
-            
+
 class TestFileProperty(TestCase):
 
     """
@@ -229,7 +388,24 @@ class TestFileProperty(TestCase):
             self.assertEqual(hfile, hfile2)
         finally:
             hfile.close()
-        
+
+    def test_property_object_id(self):
+        """ File property should have the same ObjectID as the original """
+        fname = self.mktemp()
+        hfile = File(fname, 'w')
+        try:
+            self.assertIs(hfile.id, hfile.file.id)
+        finally:
+            hfile.close()
+        # otherwise, following up with this next block would yield:
+        # ValueError: invalid file identifier
+        # (Invalid arguments to routine: Inappropriate type)
+        hfile = File(fname, 'w')
+        try:
+            self.assertEqual(hfile.id, hfile.file.id)
+        finally:
+            hfile.close()
+
     def test_close(self):
         """ All retrieved File objects are closed at the same time """
         fname = self.mktemp()
@@ -263,7 +439,7 @@ class TestClose(TestCase):
         self.assert_(fid)
         fid.close()
         self.assert_(not fid)
-      
+
     def test_closed_file(self):
         """ Trying to modify closed file raises ValueError """
         fid = File(self.mktemp(), 'w')
@@ -312,7 +488,7 @@ class TestFilename(TestCase):
             self.assertIsInstance(fid.filename, unicode)
         finally:
             fid.close()
-            
+
 class TestBackwardsCompat(TestCase):
 
     """
@@ -325,20 +501,25 @@ class TestBackwardsCompat(TestCase):
             self.assertIs(hfile.fid, hfile.id)
 
 
+class TestCloseInvalidatesOpenObjectIDs(TestCase):
 
+    """
+        Ensure that closing a file invalidates object IDs, as appropriate
+    """
 
-
-
-
-
-
-
-
-
-
-
-
-
-
-
+    def test_close(self):
+        """ Closing a file invalidates any of the file's open objects """
+        with File(self.mktemp(), 'w') as f1:
+            g1 = f1.create_group('foo')
+            self.assertTrue(bool(f1.id))
+            self.assertTrue(bool(g1.id))
+            f1.close()
+            self.assertFalse(bool(f1.id))
+            self.assertFalse(bool(g1.id))
+        with File(self.mktemp(), 'w') as f2:
+            g2 = f2.create_group('foo')
+            self.assertTrue(bool(f2.id))
+            self.assertTrue(bool(g2.id))
+            self.assertFalse(bool(f1.id))
+            self.assertFalse(bool(g1.id))
 
