@@ -1,7 +1,27 @@
+# This file is part of h5py, a Python interface to the HDF5 library.
+#
+# http://www.h5py.org
+#
+# Copyright 2008-2013 Andrew Collette and contributors
+#
+# License:  Standard 3-clause BSD; see "license.txt" for full license terms
+#           and contributor agreement.
+
+"""
+    Group test module.
+
+    Tests all methods and properties of Group objects, with the following
+    exceptions:
+
+    1. Method create_dataset is tested in module test_dataset
+"""
+import collections
 import numpy as np
+import os
 import sys
 
 from .common import ut, TestCase
+import h5py
 from h5py.highlevel import File, Group, SoftLink, HardLink, ExternalLink
 from h5py.highlevel import Dataset, Datatype
 from h5py import h5t
@@ -57,6 +77,20 @@ class TestCreate(BaseGroup):
         self.assertEqual(group.name, name)
         self.assertEqual(group.id.links.get_info(name.encode('utf8')).cset, h5t.CSET_UTF8)
 
+    def test_unicode_default(self):
+        """ Unicode names convertible to ASCII are stored as ASCII (issue 239)
+        """
+        name = u"/Hello, this is a name"
+        group = self.f.create_group(name)
+        self.assertEqual(group.name, name)
+        self.assertEqual(group.id.links.get_info(name.encode('utf8')).cset, h5t.CSET_ASCII)
+
+    def test_appropriate_low_level_id(self):
+        " Binding a group to a non-group identifier fails with ValueError "
+        dset = self.f.create_dataset('foo', [1])
+        with self.assertRaises(ValueError):
+            Group(dset.id)
+
 class TestDatasetAssignment(BaseGroup):
 
     """
@@ -142,7 +176,7 @@ class TestDelete(BaseGroup):
                 del hfile['foo']
         finally:
             hfile.close()
-            
+
 class TestOpen(BaseGroup):
 
     """
@@ -168,6 +202,36 @@ class TestOpen(BaseGroup):
         grp2 = self.f[grp.ref]
         self.assertEqual(grp2, grp)
 
+    def test_reference_numpyobj(self):
+        """ Object can be opened by numpy.object_ containing object ref
+
+        Test for issue 181, issue 202.
+        """
+        g = self.f.create_group('test')
+
+        rtype = h5py.special_dtype(ref=h5py.Reference)
+        dt = np.dtype([('a', 'i'),('b',rtype)])
+        dset = self.f.create_dataset('test_dset', (1,), dt)
+
+        dset[0] =(42,g.ref)
+        data = dset[0]
+        self.assertEqual(self.f[data[1]], g)
+
+    def test_invalid_ref(self):
+        """ Invalid region references should raise ValueError """
+
+        ref = h5py.h5r.Reference()
+
+        with self.assertRaises(ValueError):
+            self.f[ref]
+
+        self.f.create_group('x')
+        ref = self.f['x'].ref
+        del self.f['x']
+
+        with self.assertRaises(ValueError):
+            self.f[ref]
+
     # TODO: check that regionrefs also work with __getitem__
 
 class TestRepr(BaseGroup):
@@ -193,6 +257,8 @@ class BaseMapping(BaseGroup):
         self.groups = ('a','b','c','d')
         for x in self.groups:
             self.f.create_group(x)
+        self.f['x'] = h5py.SoftLink('/mongoose')
+        self.groups = self.groups + ('x',)
 
     def tearDown(self):
         if self.f:
@@ -251,7 +317,7 @@ class TestIter(BaseMapping):
             self.assertEqual(lst, [])
         finally:
             hfile.close()
-            
+
 @ut.skipIf(sys.version_info[0] != 2, "Py2")
 class TestPy2Dict(BaseMapping):
 
@@ -267,13 +333,13 @@ class TestPy2Dict(BaseMapping):
     def test_values(self):
         """ .values method """
         self.assertIsInstance(self.f.values(), list)
-        self.assertSameElements(self.f.values(), [self.f[x] for x in self.groups])
+        self.assertSameElements(self.f.values(), [self.f.get(x) for x in self.groups])
 
     def test_items(self):
         """ .items method """
         self.assertIsInstance(self.f.items(), list)
         self.assertSameElements(self.f.items(),
-            [(x, self.f[x]) for x in self.groups])
+            [(x, self.f.get(x)) for x in self.groups])
 
     def test_iterkeys(self):
         """ .iterkeys method """
@@ -282,12 +348,12 @@ class TestPy2Dict(BaseMapping):
     def test_itervalues(self):
         """ .itervalues method """
         self.assertSameElements([x for x in self.f.itervalues()],
-            [self.f[x] for x in self.groups])
+            [self.f.get(x) for x in self.groups])
 
     def test_iteritems(self):
         """ .iteritems method """
         self.assertSameElements([x for x in self.f.iteritems()],
-            [(x, self.f[x]) for x in self.groups])
+            [(x, self.f.get(x)) for x in self.groups])
 
 @ut.skipIf(sys.version_info[0] != 3, "Py3")
 class TestPy3Dict(BaseMapping):
@@ -303,7 +369,7 @@ class TestPy3Dict(BaseMapping):
     def test_values(self):
         """ .values provides a value view """
         vv = getattr(self.f, 'values')()
-        self.assertSameElements(list(vv), [self.f[x] for x in self.groups])
+        self.assertSameElements(list(vv), [self.f.get(x) for x in self.groups])
         self.assertEqual(len(vv), len(self.groups))
         with self.assertRaises(TypeError):
             b'x' in vv
@@ -311,10 +377,10 @@ class TestPy3Dict(BaseMapping):
     def test_items(self):
         """ .items provides an item view """
         iv = getattr(self.f, 'items')()
-        self.assertSameElements(list(iv), [(x,self.f[x]) for x in self.groups])
+        self.assertSameElements(list(iv), [(x,self.f.get(x)) for x in self.groups])
         self.assertEqual(len(iv), len(self.groups))
         for x in self.groups:
-            self.assertIn((x, self.f[x]), iv)
+            self.assertIn((x, self.f.get(x)), iv)
 
 class TestGet(BaseGroup):
 
@@ -429,7 +495,7 @@ class TestSoftLinks(BaseGroup):
     """
         Feature: Create and manage soft links with the high-level interface
     """
-    
+
     def test_spath(self):
         """ SoftLink path attribute """
         sl = SoftLink('/foo')
@@ -506,18 +572,69 @@ class TestExternalLinks(TestCase):
         with self.assertRaises(KeyError):
             self.f['ext']
 
+    def test_close_file(self):
+        """ Files opened by accessing external links can be closed
+
+        Issue 189.
+        """
+        self.f['ext'] = ExternalLink(self.ename, '/')
+        grp = self.f['ext']
+        f2 = grp.file
+        f2.close()
+        self.assertFalse(f2)
+
+class TestExtLinkBugs(TestCase):
+
+    """
+        Bugs: Specific regressions for external links
+    """
+
+    def test_issue_212(self):
+        """ Issue 212
+
+        Fails with:
+
+        AttributeError: 'SharedConfig' object has no attribute 'lapl'
+        """
+        def closer(x):
+            def w():
+                try:
+                    if x:
+                        x.close()
+                except IOError:
+                    pass
+            return w
+        orig_name = self.mktemp()
+        new_name = self.mktemp()
+        f = File(orig_name, 'w')
+        self.addCleanup(closer(f))
+        f.create_group('a')
+        f.close()
+
+        g = File(new_name, 'w')
+        self.addCleanup(closer(g))
+        g['link'] = ExternalLink(orig_name, '/')  # note root group
+        g.close()
+
+        h = File(new_name, 'r')
+        self.addCleanup(closer(h))
+        self.assertIsInstance(h['link']['a'], Group)
+
+
 class TestCopy(TestCase):
 
     def setUp(self):
         self.f1 = File(self.mktemp(), 'w')
         self.f2 = File(self.mktemp(), 'w')
-        
+
     def tearDown(self):
         if self.f1:
             self.f1.close()
         if self.f2:
             self.f2.close()
-            
+
+    @ut.skipIf(h5py.version.hdf5_version_tuple < (1,8,9),
+               "Bug in HDF5<1.8.8 prevents copying open dataset")
     def test_copy_path_to_path(self):
         foo = self.f1.create_group('foo')
         foo['bar'] = [1,2,3]
@@ -527,6 +644,8 @@ class TestCopy(TestCase):
         self.assertIsInstance(baz, Group)
         self.assertArrayEqual(baz['bar'], np.array([1,2,3]))
 
+    @ut.skipIf(h5py.version.hdf5_version_tuple < (1,8,9),
+               "Bug in HDF5<1.8.8 prevents copying open dataset")
     def test_copy_path_to_group(self):
         foo = self.f1.create_group('foo')
         foo['bar'] = [1,2,3]
@@ -541,6 +660,8 @@ class TestCopy(TestCase):
         self.assertIsInstance(self.f2['/foo'], Group)
         self.assertArrayEqual(self.f2['foo/bar'], np.array([1,2,3]))
 
+    @ut.skipIf(h5py.version.hdf5_version_tuple < (1,8,9),
+               "Bug in HDF5<1.8.8 prevents copying open dataset")
     def test_copy_group_to_path(self):
 
         foo = self.f1.create_group('foo')
@@ -555,6 +676,8 @@ class TestCopy(TestCase):
         self.assertIsInstance(self.f2['/foo'], Group)
         self.assertArrayEqual(self.f2['foo/bar'], np.array([1,2,3]))
 
+    @ut.skipIf(h5py.version.hdf5_version_tuple < (1,8,9),
+               "Bug in HDF5<1.8.8 prevents copying open dataset")
     def test_copy_group_to_group(self):
 
         foo = self.f1.create_group('foo')
@@ -570,6 +693,8 @@ class TestCopy(TestCase):
         self.assertIsInstance(self.f2['/foo'], Group)
         self.assertArrayEqual(self.f2['foo/bar'], np.array([1,2,3]))
 
+    @ut.skipIf(h5py.version.hdf5_version_tuple < (1,8,9),
+               "Bug in HDF5<1.8.8 prevents copying open dataset")
     def test_copy_dataset(self):
         self.f1['foo'] = [1,2,3]
         foo = self.f1['foo']
@@ -585,3 +710,160 @@ class TestCopy(TestCase):
 
         self.f2.copy(self.f1['foo'], self.f2, 'bar')
         self.assertArrayEqual(self.f2['bar'], np.array([1,2,3]))
+
+    @ut.skipIf(h5py.version.hdf5_version_tuple < (1,8,9),
+               "Bug in HDF5<1.8.8 prevents copying open dataset")
+    def test_copy_shallow(self):
+
+        foo = self.f1.create_group('foo')
+        bar = foo.create_group('bar')
+        foo['qux'] = [1,2,3]
+        bar['quux'] = [4,5,6]
+
+        self.f1.copy(foo, 'baz', shallow=True)
+        baz = self.f1['baz']
+        self.assertIsInstance(baz, Group)
+        self.assertIsInstance(baz['bar'], Group)
+        self.assertEqual(len(baz['bar']), 0)
+        self.assertArrayEqual(baz['qux'], np.array([1,2,3]))
+
+        self.f2.copy(foo, 'foo', shallow=True)
+        self.assertIsInstance(self.f2['/foo'], Group)
+        self.assertIsInstance(self.f2['foo/bar'], Group)
+        self.assertEqual(len(self.f2['foo/bar']), 0)
+        self.assertArrayEqual(self.f2['foo/qux'], np.array([1,2,3]))
+
+    @ut.skipIf(h5py.version.hdf5_version_tuple < (1,8,9),
+               "Bug in HDF5<1.8.8 prevents copying open dataset")
+    def test_copy_without_attributes(self):
+
+        self.f1['foo'] = [1,2,3]
+        foo = self.f1['foo']
+        foo.attrs['bar'] = [4,5,6]
+
+        self.f1.copy(foo, 'baz', without_attrs=True)
+        self.assertArrayEqual(self.f1['baz'], np.array([1,2,3]))
+        self.assert_('bar' not in self.f1['baz'].attrs)
+
+        self.f2.copy(foo, 'baz', without_attrs=True)
+        self.assertArrayEqual(self.f2['baz'], np.array([1,2,3]))
+        self.assert_('bar' not in self.f2['baz'].attrs)
+
+    @ut.skipIf(h5py.version.hdf5_version_tuple < (1,8,9),
+               "Bug in HDF5<1.8.8 prevents copying open dataset")
+    def test_copy_soft_links(self):
+
+        self.f1['bar'] = [1,2,3]
+        foo = self.f1.create_group('foo')
+        foo['baz'] = SoftLink('/bar')
+
+        self.f1.copy(foo, 'qux', expand_soft=True)
+        self.f2.copy(foo, 'foo', expand_soft=True)
+        del self.f1['bar']
+
+        self.assertIsInstance(self.f1['qux'], Group)
+        self.assertArrayEqual(self.f1['qux/baz'], np.array([1,2,3]))
+
+        self.assertIsInstance(self.f2['/foo'], Group)
+        self.assertArrayEqual(self.f2['foo/baz'], np.array([1,2,3]))
+
+    @ut.skipIf(h5py.version.hdf5_version_tuple < (1,8,9),
+               "Bug in HDF5<1.8.8 prevents copying open dataset")
+    def test_copy_external_links(self):
+
+        filename = self.f1.filename
+        self.f1['foo'] = [1,2,3]
+        self.f2['bar'] = ExternalLink(filename, 'foo')
+        self.f1.close()
+        self.f1 = None
+
+        self.assertArrayEqual(self.f2['bar'], np.array([1,2,3]))
+
+        self.f2.copy('bar', 'baz', expand_external=True)
+        os.unlink(filename)
+        self.assertArrayEqual(self.f2['baz'], np.array([1,2,3]))
+
+    @ut.skipIf(h5py.version.hdf5_version_tuple < (1,8,9),
+               "Bug in HDF5<1.8.8 prevents copying open dataset")
+    def test_copy_refs(self):
+
+        self.f1['foo'] = [1,2,3]
+        self.f1['bar'] = [4,5,6]
+        foo = self.f1['foo']
+        bar = self.f1['bar']
+        foo.attrs['bar'] = bar.ref
+
+        self.f1.copy(foo, 'baz', expand_refs=True)
+        self.assertArrayEqual(self.f1['baz'], np.array([1,2,3]))
+        baz_bar = self.f1['baz'].attrs['bar']
+        self.assertArrayEqual(self.f1[baz_bar], np.array([4,5,6]))
+        # The reference points to a copy of bar, not to bar itself.
+        self.assertNotEqual(self.f1[baz_bar].name, bar.name)
+
+        self.f1.copy('foo', self.f2, 'baz', expand_refs=True)
+        self.assertArrayEqual(self.f2['baz'], np.array([1,2,3]))
+        baz_bar = self.f2['baz'].attrs['bar']
+        self.assertArrayEqual(self.f2[baz_bar], np.array([4,5,6]))
+
+        self.f1.copy('/', self.f2, 'root', expand_refs=True)
+        self.assertArrayEqual(self.f2['root/foo'], np.array([1,2,3]))
+        self.assertArrayEqual(self.f2['root/bar'], np.array([4,5,6]))
+        foo_bar = self.f2['root/foo'].attrs['bar']
+        self.assertArrayEqual(self.f2[foo_bar], np.array([4,5,6]))
+        # There's only one copy of bar, which the reference points to.
+        self.assertEqual(self.f2[foo_bar], self.f2['root/bar'])
+
+
+class TestMove(BaseGroup):
+
+    """
+        Feature: Group.move moves links in a file
+    """
+
+    def test_move_hardlink(self):
+        """ Moving an object """
+        grp = self.f.create_group("X")
+        self.f.move("X", "Y")
+        self.assertEqual(self.f["Y"], grp)
+        self.f.move("Y", "new/nested/path")
+        self.assertEqual(self.f['new/nested/path'], grp)
+
+    def test_move_softlink(self):
+        """ Moving a soft link """
+        self.f['soft'] = h5py.SoftLink("relative/path")
+        self.f.move('soft', 'new_soft')
+        lnk = self.f.get('new_soft', getlink=True)
+        self.assertEqual(lnk.path, "relative/path")
+
+    def test_move_conflict(self):
+        """ Move conflict raises ValueError """
+        self.f.create_group("X")
+        self.f.create_group("Y")
+        with self.assertRaises(ValueError):
+            self.f.move("X", "Y")
+
+    def test_short_circuit(self):
+        ''' Test that a null-move works '''
+        self.f.create_group("X")
+        self.f.move("X", "X")
+
+
+class TestMutableMapping(BaseGroup):
+    '''Tests if the registration of Group as a MutableMapping
+    behaves as expected
+    '''
+    def test_resolution(self):
+        assert issubclass(Group, collections.MutableMapping)
+        grp = self.f.create_group("K")
+        assert isinstance(grp, collections.MutableMapping)
+
+    def test_validity(self):
+        '''
+        Test that the required functions are implemented.
+        '''
+        Group.__getitem__
+        Group.__setitem__
+        Group.__delitem__
+        Group.__iter__
+        Group.__len__
+
